//** Craig Laurent
#import <appkit/appkit.h>
#import <eointerface/eointerface.h>

#import "Communication.h"
#import "IconAnimator.h"
#import "InfoController.h"
#import "Preferences.h"
#import "User.h"


/* Converse - an Object that works as the main application control object.  The majority of the work is done in this object.  This object starts the DO server, displays the requested panels and handles interaction with the user. */

@interface Converse:Object <NXSenderIsInvalid>
{
	//** window objects
#ifdef IB
    id	inputView;
    id	logView;
	id	sendInButton;
#else
    Text	*inputView;
    Text	*logView;
	Button	*sendInButton;
#endif

	//**	Address window objects
#ifdef IB
	id	addressWindow;
	id	machineBrowser;
	id	machineField;
	id	addButton;
	id	removeButton;
	id	updateCheck;
	id	uniqueCheck;
#else
	Window		*addressWindow;
	NXBrowser	*machineBrowser;
	TextField	*machineField;
	Button		*addButton;
	Button		*removeButton;
	Button		*updateCheck;
	Button		*uniqueCheck;
#endif

	//** Created objects
	Preferences		*prefs;
	Communication	*commCtlr;
	NXConnection	*serverConnection;
	InfoController	*infoCtlr;
	User			*userInfo;
	NSMutableArray	*addressList;
	NSString		*filename;
	Sound			*msgSound;
	IconAnimator	*animator;
}

- init;
- free;

//** instance methods
- (Preferences*)prefs;
- (void)setPrefs:(Preferences*)newPrefs;
- (Communication*)commCtlr;
- (void)setCommCtlr:(Communication*)newComm;
- (NXConnection*)serverConnection;
- (void)setServerConnection:(NXConnection*)newServer;
- (InfoController*)infoCtlr;
- (void)setInfoCtlr:(InfoController*)newInfo;
- (User*)userInfo;
- (void)setUserInfo:(User*)newUser;
- (NSMutableArray*)addressList;
- (void)setAddressList:(NSMutableArray*)anArray;
- (NSString*)filename;
- (void)setFilename:(NSString*)string;
- (Sound*)msgSound;
- (void)setMsgSound:(Sound*)newSound;
- (void)setMsgSoundWithName:(const char*)newName;
- (IconAnimator*)animator;
- (void)setAnimator:(IconAnimator*)newIA;

//** Communication & loggin methods
- (void)createCommZone;
- sendMessage:sender;
- (NSString*)messageFromText;
- (void)logMessage:(NSString*)message fromAuthor:author alert:(BOOL)yn;
- senderIsInvalid:sender;
- (void)transferMessage:(NSString*)message;

- add:sender;
- remove:sender;
- machineClicked:sender;
- (void)refreshAddress;

- (void)loadAddressList;
- (BOOL)uniqueAddress:(User*)newAddr inRow:(unsigned int*)row;

//***** file stuff
- new:sender;
- open:sender;
- (BOOL)newOpenFilename;
- openFile:sender;
- saveAs:sender;
- (BOOL)newSaveFilename;
- save:sender;

//** menu methods
- (void)createInfoZone;
- showInfoPanel:sender;
- showPreferences:sender;
- showLogInput:sender;
- showAddress:sender;

//** delegated methods
- awakeFromNib;
- appDidBecomeActive:sender;
- appWillTerminate:sender;
- textDidGetKeys:sender isEmpty:(BOOL)flag;

//** NXBrowser delegate methods
- (int)browser:sender getNumRowsInColumn:(int)column;
- browser:sender loadCell:cell atRow:(int)row inColumn:(int)column;

@end
